@@ -36,6 +36,7 @@
 #include <sys/time.h>
 #include <sys/uio.h>
 #include <netinet/in.h>
+#include <signal.h>
 
 #include <errno.h>
 #include <netdb.h>
@@ -93,6 +94,16 @@ _download_seterr(struct downloaderr *p, int e)
     snprintf(downloadLastErrString, MAXERRSTRING, "%s", p->string);
 }
 
+/*
+ * Unset error code
+ */
+void
+_download_unseterr(void)
+{
+    downloadLastErrCode = DLERR_NONE;
+    memset(downloadLastErrString, 0, MAXERRSTRING);
+}
+
 /*
  * Set error code according to errno
  */
@@ -222,7 +233,6 @@ _download_reopen(int sd)
 conn_t *
 _download_ref(conn_t *conn)
 {
-
     ++conn->ref;
     return (conn);
 }
@@ -538,6 +548,9 @@ _download_writev(conn_t *conn, struct iovec *iov, int iovcnt)
             }
         }
         errno = 0;
+
+        /* Protect against SIGPIPE in the case of a timeout */
+        signal(SIGPIPE, SIG_IGN);
 #ifdef WITH_SSL
         if (conn->ssl != NULL)
             wlen = SSL_write(conn->ssl,
@@ -545,6 +558,8 @@ _download_writev(conn_t *conn, struct iovec *iov, int iovcnt)
         else
 #endif
             wlen = writev(conn->sd, iov, iovcnt);
+        signal(SIGPIPE, SIG_DFL);
+
         if (wlen == 0) {
             /* we consider a short write a failure */
             errno = EPIPE;
